#include <stdio.h>
#include "sim.h"
#include <linux/timeofday.h>

extern void *malloc(size_t size);

#define POLL 1 /* adjtime frequency (per interval)*/
#define PVAL 10000 /* print output frequeny (per poll) */
#define INTERVAL 1000000

struct {
	unsigned long long hardware_counter;
	double long simulator_time;
	int ppm;
} sim_data;


void simulator_data_init(long long offset, int ppm)
{
	sim_data.hardware_counter = 0;
	sim_data.simulator_time = offset;
	sim_data.ppm = ppm;
}


void increment_simulator_time()
{
	/* increment the hardware and simulator time */
	sim_data.hardware_counter += INTERVAL;
	sim_data.simulator_time += INTERVAL;
	/* add hardware skew value */
	sim_data.simulator_time += ((double long)sim_data.ppm*INTERVAL)/1000000;
}

unsigned long long simulator_get_hardware(void)
{
	return sim_data.hardware_counter;
}

unsigned long long simulator_get_time(void)
{
	return (unsigned long long)sim_data.simulator_time;
}


int main(int argc, void** argv)
{
	long long i;
	static int p;
	long long user_offset = 0;
	long user_ppm = 0;

	/* arg processing */
	if(argc > 1)
		user_offset = strtoll(argv[1],0,0);
	if(argc > 2)
		user_ppm = strtol(argv[2],0,0);

	/* setup the simulator values from the arguments*/
	simulator_data_init(user_offset, user_ppm);

	/* start the timeofday code */
	timeofday_init();


	printf("simulator time:   kernel time    hardware cycle  time offset   ppm adj\n");
	printf("------------------------------------------------------------------------\n");
	while(1) {
		i++;
		/* increment the simulator state and notify the kernel*/
		increment_simulator_time();
		timeofday_periodic_hook();

		/* every so often, do adjtimex */
		if(!(i%POLL)) {
			unsigned long long sim_time, kern_time, hardware;
			long long diff; 
			struct timespec t;
			long ppm;
			/* get the current simulator/kernel state */
			sim_time = simulator_get_time();
			hardware = simulator_get_hardware();
			kern_time = do_monotonic_clock();
			ppm = ntp_get_ppm_adjustment() >> SHIFT_USEC;

			/* calculate the offset */
			t.tv_sec = kern_time / NSEC_PER_SEC;
			diff = sim_time - kern_time;
			if(diff < 0)
				diff = -(-diff/ NSEC_PER_USEC);
			else
				diff /= NSEC_PER_USEC;

			/* update kernel ntp satemachine with the current offset */
			adjtimex_hardupdate((long)diff, t);

			/* print simulator state every so often */
			if (!(p++%PVAL))
				printf("%llu ns:  %llu ns  %llu cyc  %lld us  %ld ppm\n", sim_time, kern_time, hardware, diff, ppm);
		}
	}	
	return 0;
}



